/*
Section:  4) 1NF to BCNF
Script:   10) Getting Data Into BCNF -- Final
Basics:   Going back to 1NF, we want to get to BCNF.
Notes:    
*/

/*
The difference between BCNF and 2NF/3NF is that BCNF is
a bit "stricter" with whether attributes really belong to
a particular element.  When there is only one candidate
key, BCNF = 3NF, so we're going to focus on the Song table.

Bonus change:  we're switching the primary key to AlbumName/TrackNumber
rather than the original AlbumName/SongName, just to make the
difference between BCNF and 3NF clearer.

1NF structure:
Song
	- Year
	- Type { Studio, Live }
	- AlbumName (PK, AK2)
	- Label
	- LabelHeadquarters
	- TrackNumber (PK)
	- SongName (AK2)
	- SongNameHasNumeral <-- NEW

Candidate keys:
{ AlbumName, TrackNumber }
{ AlbumName, SongName }

1)  Determine all functional dependencies.

f1) AlbumName --> Year,  Type, Label
f2) Label --> LabelHeadquarters
f3) SongName --> SongNameHasNumeral
f4) AlbumName, TrackNumber --> SongName
f5) AlbumName, SongName --> TrackNumber

2)  Normalize this structure into Boyce-Codd Normal Form.

Functional dependencies f4 and f5 are good--the left-hand side of
each makes up a candidate key.  Unfortunately, f1-f3 are bad.  To fix
them, we break this down dependency by dependency.  Let's start with f2
because it's simple.

Label:	(Label, LabelHeadquarters)
	Functional Dependency:
		f2) Label --> LabelHeadquarters
Song:	(Year, Type, AlbumName, Label, TrackNumber, SongName, SongNameHasNumeral)
	Functional Dependencies:
		f1) AlbumName --> Year,  Type, Label
		f3) SongName --> SongNameHasNumeral
		f4) AlbumName, TrackNumber --> SongName
		f5) AlbumName, SongName --> TrackNumber

We know this isn't enough, so let's take care of f3 next.

Label:	(Label, LabelHeadquarters)
	Functional Dependency:
		f2) Label --> LabelHeadquarters
SongName:  (SongName, SongNameHasNumeral)
	Functional Dependency:
		f3)  SongName --> SongNameHasNumeral
Song:	(Year, Type, AlbumName, Label, TrackNumber, SongName)
	Functional Dependencies:
		f1) AlbumName --> Year,  Type, Label
		f4) AlbumName, TrackNumber --> SongName
		f5) AlbumName, SongName --> TrackNumber

Close, but we have one more functional dependency to solve.
Label:	(Label, LabelHeadquarters)
	Functional Dependency:
		f2) Label --> LabelHeadquarters
SongName:  (SongName, SongNameHasNumeral)
	Functional Dependency:
		f3)  SongName --> SongNameHasNumeral
Album:  (AlbumName, Year, Type, Label)
	Functional Dependency:
		f1) AlbumName --> Year,  Type, Label
Song:	(AlbumName, TrackNumber, SongName)
	Functional Dependencies:		
		f4) AlbumName, TrackNumber --> SongName
		f5) AlbumName, SongName --> TrackNumber

Final BCNF structure:
Label
	- Name
	- Headquarters

Album
	- Year
	- Type
	- Name
	- LabelName

Song
	- AlbumName
	- TrackNumber
	- SongName

SongName
	- SongName
	- SongNameHasNumeral

*/
